@@ -16,9 +16,7 @@ use crate::sql_types::{self, HasSqlType, TypeMetadata};
 #[diesel_derives::__diesel_public_if(
     feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes"
 )]
-pub(crate) use self::private::{
-    DieselReserveSpecialization, HasBindCollector, HasRawValue, TrustedBackend,
-};
+pub(crate) use self::private::{DieselReserveSpecialization, TrustedBackend};
 
 /// A database backend
 ///
@@ -37,40 +35,24 @@ pub(crate) use self::private::{
 /// connection is implemented.
 /// For example, the `Pg` backend does not assume that `libpq` is being used.
 /// Implementations of this trait can and should care about details of the wire
-/// protocol used to communicated with the database.
+/// protocol used to communicate with the database.
 ///
 /// Implementing support for a new backend is a complex topic and depends on the
-/// details how the newly implemented backend may communicate with diesel. As of this
+/// details how the newly implemented backend may communicate with diesel. As of this,
 /// we cannot provide concrete examples here and only present a general outline of
 /// the required steps. Existing backend implementations provide a good starting point
-/// to see how certain things are solve for other backend implementations.
+/// to see how certain things are solved for other backend implementations.
 ///
 /// Types implementing `Backend` should generally be zero sized structs.
 ///
-/// To implement the `Backend` trait you need to:
+/// To implement the `Backend` trait, you need to:
 ///
 /// * Specify how a query should be build from string parts by providing a [`QueryBuilder`]
 /// matching your backend
 /// * Specify the bind value format used by your database connection library by providing
-/// a [`BindCollector`] matching your backend via
-#[cfg_attr(
-    feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes",
-    doc = "[`HasBindCollector`]"
-)]
-#[cfg_attr(
-    not(feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes"),
-    doc = "`HasBindCollector`"
-)]
-/// * Specify  how values are receive from the database by providing a corresponding raw value
-/// definition via
-#[cfg_attr(
-    feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes",
-    doc = "[`HasRawValue`]"
-)]
-#[cfg_attr(
-    not(feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes"),
-    doc = "`HasRawValue`"
-)]
+/// a [`BindCollector`](crate::query_builder::bind_collector::BindCollector) matching your backend
+/// * Specify how values are received from the database by providing a corresponding raw value
+/// definition
 /// * Control sql dialect specific parts of diesels query dsl implementation by providing a
 /// matching [`SqlDialect`] implementation
 /// * Implement [`TypeMetadata`] to specify how your backend identifies types
@@ -100,7 +82,7 @@ pub(crate) use self::private::{
 /// [`QueryFragment`]: crate::query_builder::QueryFragment
 pub trait Backend
 where
-    Self: Sized + SqlDialect,
+    Self: Sized + SqlDialect + TypeMetadata,
     Self: HasSqlType<sql_types::SmallInt>,
     Self: HasSqlType<sql_types::Integer>,
     Self: HasSqlType<sql_types::BigInt>,
@@ -111,29 +93,41 @@ where
     Self: HasSqlType<sql_types::Date>,
     Self: HasSqlType<sql_types::Time>,
     Self: HasSqlType<sql_types::Timestamp>,
-    Self: for<'a> HasRawValue<'a>,
-    Self: for<'a> HasBindCollector<'a>,
 {
     /// The concrete [`QueryBuilder`] implementation for this backend.
     type QueryBuilder: QueryBuilder<Self>;
+
+    /// The actual type given to [`FromSql`], with lifetimes applied. This type
+    /// should not be used directly.
+    ///
+    /// [`FromSql`]: crate::deserialize::FromSql
+    type RawValue<'a>;
+
+    /// The concrete [`BindCollector`](crate::query_builder::bind_collector::BindCollector)
+    /// implementation for this backend.
+    ///
+    /// Most backends should use [`RawBytesBindCollector`].
+    ///
+    /// [`RawBytesBindCollector`]: crate::query_builder::bind_collector::RawBytesBindCollector
+    type BindCollector<'a>: crate::query_builder::bind_collector::BindCollector<'a, Self> + 'a;
 }
 
-/// A helper type to get the raw representation of a database type given to
-/// [`FromSql`]. Equivalent to `<DB as Backend>::RawValue<'a>`.
-///
-/// [`FromSql`]: crate::deserialize::FromSql
-pub type RawValue<'a, DB> = <DB as HasRawValue<'a>>::RawValue;
+#[doc(hidden)]
+#[cfg(all(feature = "with-deprecated", not(feature = "without-deprecated")))]
+#[deprecated(note = "Use `Backend::RawValue` directly")]
+pub type RawValue<'a, DB> = <DB as Backend>::RawValue<'a>;
 
-/// A helper type to get the bind collector for a database backend.
-/// Equivalent to `<DB as HasBindCollector<'a>>::BindCollector<'a>`j
-pub type BindCollector<'a, DB> = <DB as HasBindCollector<'a>>::BindCollector;
+#[doc(hidden)]
+#[cfg(all(feature = "with-deprecated", not(feature = "without-deprecated")))]
+#[deprecated(note = "Use `Backend::BindCollector` directly")]
+pub type BindCollector<'a, DB> = <DB as Backend>::BindCollector<'a>;
 
 /// This trait provides various options to configure the
 /// generated SQL for a specific backend.
 ///
 /// Accessing anything from this trait is considered to be part of the
 /// public API. Implementing this trait is not considered to be part of
-/// diesels public API, as future versions of diesel may add additional
+/// diesel's public API, as future versions of diesel may add additional
 /// associated constants here.
 ///
 /// Each associated type is used to configure the behaviour
@@ -147,9 +141,9 @@ pub type BindCollector<'a, DB> = <DB as HasBindCollector<'a>>::BindCollector;
     doc = "See the [`sql_dialect`] module for options provided by diesel out of the box."
 )]
 pub trait SqlDialect: self::private::TrustedBackend {
-    /// Configures how this backends supports `RETURNING` clauses
+    /// Configures how this backend supports `RETURNING` clauses
     ///
-    /// This allows backends to opt in  `RETURNING` clause support and to
+    /// This allows backends to opt in `RETURNING` clause support and to
     /// provide a custom [`QueryFragment`](crate::query_builder::QueryFragment)
     #[cfg_attr(
         feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes",
@@ -312,6 +306,17 @@ pub trait SqlDialect: self::private::TrustedBackend {
         doc = "See [`sql_dialect::select_statement_syntax`] for provided default implementations"
     )]
     type SelectStatementSyntax;
+
+    /// Configures how this backend structures `SELECT` queries
+    ///
+    /// This allows backends to provide custom [`QueryFragment`](crate::query_builder::QueryFragment)
+    /// implementations for [`Alias<T>`](crate::query_source::Alias)
+    ///
+    #[cfg_attr(
+        feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes",
+        doc = "See [`sql_dialect::alias_syntax`] for provided default implementations"
+    )]
+    type AliasSyntax;
 }
 
 /// This module contains all options provided by diesel to configure the [`SqlDialect`] trait.
@@ -321,7 +326,7 @@ pub trait SqlDialect: self::private::TrustedBackend {
 // * The backend fixed/introduced a bug that requires special handling
 // * We got some edge case wrong with sharing the implementation between backends
 //
-// By not exposing these types publically we are able to change the exact definitions later on
+// By not exposing these types publicly we are able to change the exact definitions later on
 // as users cannot write trait bounds that ensure that a specific type is used in place of
 // an existing associated type.
 #[diesel_derives::__diesel_public_if(
@@ -343,7 +348,7 @@ pub(crate) mod sql_dialect {
     #[diesel_derives::__diesel_public_if(
         feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes"
     )]
-    pub(crate) mod on_conflict_clause {
+    pub mod on_conflict_clause {
         /// A marker trait indicating if a `ON CONFLICT` clause is supported or not
         ///
         /// If you use a custom type to specify specialized support for `ON CONFLICT` clauses
@@ -351,6 +356,13 @@ pub(crate) mod sql_dialect {
         /// `QueryFragment` implementations
         pub trait SupportsOnConflictClause {}
 
+        /// A marker trait indicating if a `ON CONFLICT (...) DO UPDATE ... [WHERE ...]` clause is supported or not
+        pub trait SupportsOnConflictClauseWhere {}
+
+        /// A marker trait indicating whether the on conflict clause implementation
+        /// is mostly like postgresql
+        pub trait PgLikeOnConflictClause: SupportsOnConflictClause {}
+
         /// This marker type indicates that `ON CONFLICT` clauses are not supported for this backend
         #[derive(Debug, Copy, Clone)]
         pub struct DoesNotSupportOnConflictClause;
@@ -361,10 +373,10 @@ pub(crate) mod sql_dialect {
     #[diesel_derives::__diesel_public_if(
         feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes"
     )]
-    pub(crate) mod returning_clause {
+    pub mod returning_clause {
         /// A marker trait indicating if a `RETURNING` clause is supported or not
         ///
-        /// If you use custom type to specify specialized support for `RETURNING` clauses
+        /// If you use a custom type to specify specialized support for `RETURNING` clauses
         /// implementing this trait opts in supporting `RETURNING` clause syntax
         pub trait SupportsReturningClause {}
 
@@ -385,7 +397,7 @@ pub(crate) mod sql_dialect {
     #[diesel_derives::__diesel_public_if(
         feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes"
     )]
-    pub(crate) mod default_keyword_for_insert {
+    pub mod default_keyword_for_insert {
         /// A marker trait indicating if a `DEFAULT` like expression
         /// is supported as part of `INSERT INTO` clauses to indicate
         /// that a default value should be inserted at a specific position
@@ -402,7 +414,7 @@ pub(crate) mod sql_dialect {
         pub struct IsoSqlDefaultKeyword;
 
         /// Indicates that a backend does not support `DEFAULT` value
-        /// expressions0for `INSERT INTO` statements
+        /// expressions for `INSERT INTO` statements
         #[derive(Debug, Copy, Clone)]
         pub struct DoesNotSupportDefaultKeyword;
 
@@ -414,7 +426,7 @@ pub(crate) mod sql_dialect {
     #[diesel_derives::__diesel_public_if(
         feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes"
     )]
-    pub(crate) mod batch_insert_support {
+    pub mod batch_insert_support {
         /// A marker trait indicating if batch insert statements
         /// are supported for this backend or not
         pub trait SupportsBatchInsert {}
@@ -422,7 +434,7 @@ pub(crate) mod sql_dialect {
         /// Indicates that this backend does not support batch
         /// insert statements.
         /// In this case diesel will emulate batch insert support
-        /// by inserting each row on it's own
+        /// by inserting each row on its own
         #[derive(Debug, Copy, Clone)]
         pub struct DoesNotSupportBatchInsert;
 
@@ -439,7 +451,7 @@ pub(crate) mod sql_dialect {
     #[diesel_derives::__diesel_public_if(
         feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes"
     )]
-    pub(crate) mod concat_clause {
+    pub mod concat_clause {
 
         /// Indicates that this backend uses the
         /// `||` operator to select a concatenation
@@ -453,7 +465,7 @@ pub(crate) mod sql_dialect {
     #[diesel_derives::__diesel_public_if(
         feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes"
     )]
-    pub(crate) mod default_value_clause {
+    pub mod default_value_clause {
 
         /// Indicates that this backend uses the
         /// `DEFAULT VALUES` syntax to specify
@@ -482,7 +494,7 @@ pub(crate) mod sql_dialect {
     #[diesel_derives::__diesel_public_if(
         feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes"
     )]
-    pub(crate) mod exists_syntax {
+    pub mod exists_syntax {
 
         /// Indicates that this backend
         /// treats `EXIST()` as function
@@ -496,7 +508,7 @@ pub(crate) mod sql_dialect {
     #[diesel_derives::__diesel_public_if(
         feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes"
     )]
-    pub(crate) mod array_comparison {
+    pub mod array_comparison {
 
         /// Indicates that this backend requires a single bind
         /// per array element in `IN()` and `NOT IN()` expression
@@ -509,37 +521,30 @@ pub(crate) mod sql_dialect {
     #[diesel_derives::__diesel_public_if(
         feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes"
     )]
-    pub(crate) mod select_statement_syntax {
+    pub mod select_statement_syntax {
         /// Indicates that this backend uses the default
         /// ANSI select statement structure
         #[derive(Debug, Copy, Clone)]
         pub struct AnsiSqlSelectStatement;
     }
+
+    /// This module contains all reusable options to configure
+    /// [`SqlDialect::AliasSyntax`]
+    #[diesel_derives::__diesel_public_if(
+        feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes"
+    )]
+    pub mod alias_syntax {
+        /// Indicates that this backend uses `table AS alias` for
+        /// defining table aliases
+        #[derive(Debug, Copy, Clone)]
+        pub struct AsAliasSyntax;
+    }
 }
 
 // These traits are not part of the public API
 // because we want to replace them by with an associated type
 // in the child trait later if GAT's are finally stable
-mod private {
-    use super::TypeMetadata;
-
-    /// The raw representation of a database value given to `FromSql`.
-    ///
-    /// This trait is separate from [`Backend`](super::Backend) to imitate `type RawValue<'a>`. It
-    /// should only be referenced directly by implementors. Users of this type
-    /// should instead use the [`RawValue`](super::RawValue) helper type instead.
-    #[cfg_attr(
-        doc_cfg,
-        doc(cfg(feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes"))
-    )]
-    pub trait HasRawValue<'a> {
-        /// The actual type given to [`FromSql`], with lifetimes applied. This type
-        /// should not be used directly. Use the [`RawValue`](super::RawValue)
-        /// helper type instead.
-        ///
-        /// [`FromSql`]: crate::deserialize::FromSql
-        type RawValue;
-    }
+pub(crate) mod private {
 
     /// This is a marker trait which indicates that
     /// diesel may specialize a certain [`QueryFragment`]
@@ -547,7 +552,7 @@ mod private {
     /// suggests adding this a bound to a type implementing `Backend`
     /// consider adding the following bound instead
     /// `YourQueryType: QueryFragment<DB>` (the concrete bound
-    /// is likely mentioned by rustc as part of a `note: …`
+    /// is likely mentioned by rustc as part of a `note: …`)
     ///
     /// For any user implementing a custom backend: You likely want to implement
     /// this trait for your custom backend type to opt in the existing [`QueryFragment`] impls in diesel.
@@ -557,38 +562,17 @@ mod private {
     ///
     /// [`QueryFragment`]: crate::query_builder::QueryFragment
     #[cfg_attr(
-        doc_cfg,
+        docsrs,
         doc(cfg(feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes"))
     )]
     pub trait DieselReserveSpecialization {}
 
-    /// The bind collector type used to collect query binds for this backend
-    ///
-    /// This trait is separate from [`Backend`](super::Backend) to imitate `type BindCollector<'a>`. It
-    /// should only be referenced directly by implementors. Users of this type
-    /// should instead use the [`BindCollector`] helper type instead.
-    ///
-    /// [`BindCollector`]: super::BindCollector
-    #[cfg_attr(
-        doc_cfg,
-        doc(cfg(feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes"))
-    )]
-    pub trait HasBindCollector<'a>: TypeMetadata + Sized {
-        /// The concrete [`BindCollector`](crate::query_builder::bind_collector::BindCollector)
-        /// implementation for this backend.
-        ///
-        /// Most backends should use [`RawBytesBindCollector`].
-        ///
-        /// [`RawBytesBindCollector`]: crate::query_builder::bind_collector::RawBytesBindCollector
-        type BindCollector: crate::query_builder::bind_collector::BindCollector<'a, Self> + 'a;
-    }
-
-    /// This trait just indicates that noone implements
+    /// This trait just indicates that none implements
     /// [`SqlDialect`](super::SqlDialect) without enabling the
     /// `i-implement-a-third-party-backend-and-opt-into-breaking-changes`
     /// feature flag.
     #[cfg_attr(
-        doc_cfg,
+        docsrs,
         doc(cfg(feature = "i-implement-a-third-party-backend-and-opt-into-breaking-changes"))
     )]
     pub trait TrustedBackend {}
