@@ -1,19 +1,92 @@
 extern crate chrono;
+
 use self::chrono::{DateTime, Local, NaiveDate, NaiveDateTime, NaiveTime, TimeZone, Utc};
 
-use crate::backend;
+use crate::backend::Backend;
 use crate::deserialize::{self, FromSql};
 use crate::serialize::{self, IsNull, Output, ToSql};
 use crate::sql_types::{Date, Time, Timestamp, TimestamptzSqlite};
 use crate::sqlite::Sqlite;
 
-const SQLITE_DATE_FORMAT: &str = "%F";
+/// Warning to future editors:
+/// Changes in the following formats need to be kept in sync
+/// with the formats of the ["time"](super::time) module.
+/// We do not need a distinction between whole second and
+/// subsecond since %.f will only print the dot if needed.
+/// We always print as many subsecond as his given to us,
+/// this means the subsecond part can be 3, 6 or 9 digits.
+const DATE_FORMAT: &str = "%F";
+
+const ENCODE_TIME_FORMAT: &str = "%T%.f";
+
+const TIME_FORMATS: [&str; 9] = [
+    // Most likely formats
+    "%T%.f", "%T", // All other valid formats in order of increasing specificity
+    "%R", "%RZ", "%R%:z", "%TZ", "%T%:z", "%T%.fZ", "%T%.f%:z",
+];
+
+const ENCODE_NAIVE_DATETIME_FORMAT: &str = "%F %T%.f";
+
+const ENCODE_DATETIME_FORMAT: &str = "%F %T%.f%:z";
+
+const NAIVE_DATETIME_FORMATS: [&str; 18] = [
+    // Most likely formats
+    "%F %T%.f",
+    "%F %T%.f%:z",
+    "%F %T",
+    "%F %T%:z",
+    // All other formats in order of increasing specificity
+    "%F %R",
+    "%F %RZ",
+    "%F %R%:z",
+    "%F %TZ",
+    "%F %T%.fZ",
+    "%FT%R",
+    "%FT%RZ",
+    "%FT%R%:z",
+    "%FT%T",
+    "%FT%TZ",
+    "%FT%T%:z",
+    "%FT%T%.f",
+    "%FT%T%.fZ",
+    "%FT%T%.f%:z",
+];
+
+const DATETIME_FORMATS: [&str; 12] = [
+    // Most likely formats
+    "%F %T%.f%:z",
+    "%F %T%:z",
+    // All other formats in order of increasing specificity
+    "%F %RZ",
+    "%F %R%:z",
+    "%F %TZ",
+    "%F %T%.fZ",
+    "%FT%RZ",
+    "%FT%R%:z",
+    "%FT%TZ",
+    "%FT%T%:z",
+    "%FT%T%.fZ",
+    "%FT%T%.f%:z",
+];
+
+fn parse_julian(julian_days: f64) -> Option<NaiveDateTime> {
+    const EPOCH_IN_JULIAN_DAYS: f64 = 2_440_587.5;
+    const SECONDS_IN_DAY: f64 = 86400.0;
+    let timestamp = (julian_days - EPOCH_IN_JULIAN_DAYS) * SECONDS_IN_DAY;
+    #[allow(clippy::cast_possible_truncation)] // we want to truncate
+    let seconds = timestamp.trunc() as i64;
+    // that's not true, `fract` is always > 0
+    #[allow(clippy::cast_sign_loss, clippy::cast_possible_truncation)]
+    let nanos = (timestamp.fract() * 1E9) as u32;
+    #[allow(deprecated)] // otherwise we would need to bump our minimal chrono version
+    NaiveDateTime::from_timestamp_opt(seconds, nanos)
+}
 
 #[cfg(all(feature = "sqlite", feature = "chrono"))]
 impl FromSql<Date, Sqlite> for NaiveDate {
-    fn from_sql(value: backend::RawValue<'_, Sqlite>) -> deserialize::Result<Self> {
+    fn from_sql(value: <Sqlite as Backend>::RawValue<'_>) -> deserialize::Result<Self> {
         value
-            .parse_string(|s| Self::parse_from_str(s, SQLITE_DATE_FORMAT))
+            .parse_string(|s| Self::parse_from_str(s, DATE_FORMAT))
             .map_err(Into::into)
     }
 }
@@ -21,28 +94,22 @@ impl FromSql<Date, Sqlite> for NaiveDate {
 #[cfg(all(feature = "sqlite", feature = "chrono"))]
 impl ToSql<Date, Sqlite> for NaiveDate {
     fn to_sql<'b>(&'b self, out: &mut Output<'b, '_, Sqlite>) -> serialize::Result {
-        out.set_value(self.format(SQLITE_DATE_FORMAT).to_string());
+        out.set_value(self.format(DATE_FORMAT).to_string());
         Ok(IsNull::No)
     }
 }
 
 #[cfg(all(feature = "sqlite", feature = "chrono"))]
 impl FromSql<Time, Sqlite> for NaiveTime {
-    fn from_sql(value: backend::RawValue<'_, Sqlite>) -> deserialize::Result<Self> {
+    fn from_sql(value: <Sqlite as Backend>::RawValue<'_>) -> deserialize::Result<Self> {
         value.parse_string(|text| {
-            let valid_time_formats = &[
-                // Most likely
-                "%T%.f", // All other valid formats in order of documentation
-                "%R", "%RZ", "%T%.fZ", "%R%:z", "%T%.f%:z",
-            ];
-
-            for format in valid_time_formats {
+            for format in TIME_FORMATS {
                 if let Ok(time) = Self::parse_from_str(text, format) {
                     return Ok(time);
                 }
             }
 
-            Err(format!("Invalid time {}", text).into())
+            Err(format!("Invalid time {text}").into())
         })
     }
 }
@@ -50,50 +117,28 @@ impl FromSql<Time, Sqlite> for NaiveTime {
 #[cfg(all(feature = "sqlite", feature = "chrono"))]
 impl ToSql<Time, Sqlite> for NaiveTime {
     fn to_sql<'b>(&'b self, out: &mut Output<'b, '_, Sqlite>) -> serialize::Result {
-        out.set_value(self.format("%T%.f").to_string());
+        out.set_value(self.format(ENCODE_TIME_FORMAT).to_string());
         Ok(IsNull::No)
     }
 }
 
 #[cfg(all(feature = "sqlite", feature = "chrono"))]
 impl FromSql<Timestamp, Sqlite> for NaiveDateTime {
-    fn from_sql(value: backend::RawValue<'_, Sqlite>) -> deserialize::Result<Self> {
+    fn from_sql(value: <Sqlite as Backend>::RawValue<'_>) -> deserialize::Result<Self> {
         value.parse_string(|text| {
-            let sqlite_datetime_formats = &[
-                // Most likely format
-                "%F %T%.f",
-                // Other formats in order of appearance in docs
-                "%F %R",
-                "%F %RZ",
-                "%F %R%:z",
-                "%F %T%.fZ",
-                "%F %T%.f%:z",
-                "%FT%R",
-                "%FT%RZ",
-                "%FT%R%:z",
-                "%FT%T%.f",
-                "%FT%T%.fZ",
-                "%FT%T%.f%:z",
-            ];
-
-            for format in sqlite_datetime_formats {
+            for format in NAIVE_DATETIME_FORMATS {
                 if let Ok(dt) = Self::parse_from_str(text, format) {
                     return Ok(dt);
                 }
             }
 
             if let Ok(julian_days) = text.parse::<f64>() {
-                let epoch_in_julian_days = 2_440_587.5;
-                let seconds_in_day = 86400.0;
-                let timestamp = (julian_days - epoch_in_julian_days) * seconds_in_day;
-                let seconds = timestamp as i64;
-                let nanos = (timestamp.fract() * 1E9) as u32;
-                if let Some(timestamp) = Self::from_timestamp_opt(seconds, nanos) {
+                if let Some(timestamp) = parse_julian(julian_days) {
                     return Ok(timestamp);
                 }
             }
 
-            Err(format!("Invalid datetime {}", text).into())
+            Err(format!("Invalid datetime {text}").into())
         })
     }
 }
@@ -101,48 +146,28 @@ impl FromSql<Timestamp, Sqlite> for NaiveDateTime {
 #[cfg(all(feature = "sqlite", feature = "chrono"))]
 impl ToSql<Timestamp, Sqlite> for NaiveDateTime {
     fn to_sql<'b>(&'b self, out: &mut Output<'b, '_, Sqlite>) -> serialize::Result {
-        out.set_value(self.format("%F %T%.f").to_string());
+        out.set_value(self.format(ENCODE_NAIVE_DATETIME_FORMAT).to_string());
         Ok(IsNull::No)
     }
 }
 
 #[cfg(all(feature = "sqlite", feature = "chrono"))]
 impl FromSql<TimestamptzSqlite, Sqlite> for NaiveDateTime {
-    fn from_sql(value: backend::RawValue<'_, Sqlite>) -> deserialize::Result<Self> {
+    fn from_sql(value: <Sqlite as Backend>::RawValue<'_>) -> deserialize::Result<Self> {
         value.parse_string(|text| {
-            let sqlite_datetime_formats = &[
-                "%F %RZ",
-                "%F %R%:z",
-                "%F %TZ",
-                "%F %T%:z",
-                "%F %T%.fZ",
-                "%F %T%.f%:z",
-                "%FT%RZ",
-                "%FT%R%:z",
-                "%FT%TZ",
-                "%FT%T%:z",
-                "%FT%T%.fZ",
-                "%FT%T%.f%:z",
-            ];
-
-            for format in sqlite_datetime_formats {
+            for format in NAIVE_DATETIME_FORMATS {
                 if let Ok(dt) = Self::parse_from_str(text, format) {
                     return Ok(dt);
                 }
             }
 
             if let Ok(julian_days) = text.parse::<f64>() {
-                let epoch_in_julian_days = 2_440_587.5;
-                let seconds_in_day = 86400.0;
-                let timestamp = (julian_days - epoch_in_julian_days) * seconds_in_day;
-                let seconds = timestamp as i64;
-                let nanos = (timestamp.fract() * 1E9) as u32;
-                if let Some(timestamp) = Self::from_timestamp_opt(seconds, nanos) {
+                if let Some(timestamp) = parse_julian(julian_days) {
                     return Ok(timestamp);
                 }
             }
 
-            Err(format!("Invalid datetime {}", text).into())
+            Err(format!("Invalid datetime {text}").into())
         })
     }
 }
@@ -150,23 +175,51 @@ impl FromSql<TimestamptzSqlite, Sqlite> for NaiveDateTime {
 #[cfg(all(feature = "sqlite", feature = "chrono"))]
 impl ToSql<TimestamptzSqlite, Sqlite> for NaiveDateTime {
     fn to_sql<'b>(&'b self, out: &mut Output<'b, '_, Sqlite>) -> serialize::Result {
-        out.set_value(self.format("%F %T%.f+00:00").to_string());
+        out.set_value(self.format(ENCODE_NAIVE_DATETIME_FORMAT).to_string());
         Ok(IsNull::No)
     }
 }
 
 #[cfg(all(feature = "sqlite", feature = "chrono"))]
 impl FromSql<TimestamptzSqlite, Sqlite> for DateTime<Utc> {
-    fn from_sql(value: backend::RawValue<'_, Sqlite>) -> deserialize::Result<Self> {
+    fn from_sql(value: <Sqlite as Backend>::RawValue<'_>) -> deserialize::Result<Self> {
+        // First try to parse the timezone
+        if let Ok(dt) = value.parse_string(|text| {
+            for format in DATETIME_FORMATS {
+                if let Ok(dt) = DateTime::parse_from_str(text, format) {
+                    return Ok(dt.with_timezone(&Utc));
+                }
+            }
+
+            Err(())
+        }) {
+            return Ok(dt);
+        }
+
+        // Fallback on assuming Utc
         let naive_date_time =
             <NaiveDateTime as FromSql<TimestamptzSqlite, Sqlite>>::from_sql(value)?;
-        Ok(DateTime::from_utc(naive_date_time, Utc))
+        Ok(Utc.from_utc_datetime(&naive_date_time))
     }
 }
 
 #[cfg(all(feature = "sqlite", feature = "chrono"))]
 impl FromSql<TimestamptzSqlite, Sqlite> for DateTime<Local> {
-    fn from_sql(value: backend::RawValue<'_, Sqlite>) -> deserialize::Result<Self> {
+    fn from_sql(value: <Sqlite as Backend>::RawValue<'_>) -> deserialize::Result<Self> {
+        // First try to parse the timezone
+        if let Ok(dt) = value.parse_string(|text| {
+            for format in DATETIME_FORMATS {
+                if let Ok(dt) = DateTime::parse_from_str(text, format) {
+                    return Ok(dt.with_timezone(&Local));
+                }
+            }
+
+            Err(())
+        }) {
+            return Ok(dt);
+        }
+
+        // Fallback on assuming Local
         let naive_date_time =
             <NaiveDateTime as FromSql<TimestamptzSqlite, Sqlite>>::from_sql(value)?;
         Ok(Local::from_utc_datetime(&Local, &naive_date_time))
@@ -176,12 +229,15 @@ impl FromSql<TimestamptzSqlite, Sqlite> for DateTime<Local> {
 #[cfg(all(feature = "sqlite", feature = "chrono"))]
 impl<TZ: TimeZone> ToSql<TimestamptzSqlite, Sqlite> for DateTime<TZ> {
     fn to_sql<'b>(&'b self, out: &mut Output<'b, '_, Sqlite>) -> serialize::Result {
-        out.set_value(self.naive_utc().format("%F %T%.f+00:00").to_string());
+        // Converting to UTC ensures consistency
+        let dt_utc = self.with_timezone(&Utc);
+        out.set_value(dt_utc.format(ENCODE_DATETIME_FORMAT).to_string());
         Ok(IsNull::No)
     }
 }
 
 #[cfg(test)]
+#[allow(clippy::unwrap_used)]
 mod tests {
     extern crate chrono;
     extern crate dotenvy;
@@ -197,14 +253,17 @@ mod tests {
     use crate::sql_types::{Text, Time, Timestamp, TimestamptzSqlite};
     use crate::test_helpers::connection;
 
-    sql_function!(fn datetime(x: Text) -> Timestamp);
-    sql_function!(fn time(x: Text) -> Time);
-    sql_function!(fn date(x: Text) -> Date);
+    define_sql_function!(fn datetime(x: Text) -> Timestamp);
+    define_sql_function!(fn time(x: Text) -> Time);
+    define_sql_function!(fn date(x: Text) -> Date);
 
     #[test]
     fn unix_epoch_encodes_correctly() {
         let connection = &mut connection();
-        let time = NaiveDate::from_ymd(1970, 1, 1).and_hms(0, 0, 0);
+        let time = NaiveDate::from_ymd_opt(1970, 1, 1)
+            .unwrap()
+            .and_hms_opt(0, 0, 0)
+            .unwrap();
         let query = select(datetime("1970-01-01 00:00:00.000000").eq(time));
         assert_eq!(Ok(true), query.get_result(connection));
     }
@@ -212,7 +271,10 @@ mod tests {
     #[test]
     fn unix_epoch_decodes_correctly_in_all_possible_formats() {
         let connection = &mut connection();
-        let time = NaiveDate::from_ymd(1970, 1, 1).and_hms(0, 0, 0);
+        let time = NaiveDate::from_ymd_opt(1970, 1, 1)
+            .unwrap()
+            .and_hms_opt(0, 0, 0)
+            .unwrap();
         let valid_epoch_formats = vec![
             "1970-01-01 00:00",
             "1970-01-01 00:00:00",
@@ -267,11 +329,11 @@ mod tests {
     #[test]
     fn times_relative_to_now_encode_correctly() {
         let connection = &mut connection();
-        let time = Utc::now().naive_utc() + Duration::seconds(60);
+        let time = Utc::now().naive_utc() + Duration::try_seconds(60).unwrap();
         let query = select(now.lt(time));
         assert_eq!(Ok(true), query.get_result(connection));
 
-        let time = Utc::now().naive_utc() - Duration::seconds(600);
+        let time = Utc::now().naive_utc() - Duration::try_seconds(600).unwrap();
         let query = select(now.gt(time));
         assert_eq!(Ok(true), query.get_result(connection));
     }
@@ -280,15 +342,15 @@ mod tests {
     fn times_of_day_encode_correctly() {
         let connection = &mut connection();
 
-        let midnight = NaiveTime::from_hms(0, 0, 0);
+        let midnight = NaiveTime::from_hms_opt(0, 0, 0).unwrap();
         let query = select(time("00:00:00.000000").eq(midnight));
         assert!(query.get_result::<bool>(connection).unwrap());
 
-        let noon = NaiveTime::from_hms(12, 0, 0);
+        let noon = NaiveTime::from_hms_opt(12, 0, 0).unwrap();
         let query = select(time("12:00:00.000000").eq(noon));
         assert!(query.get_result::<bool>(connection).unwrap());
 
-        let roughly_half_past_eleven = NaiveTime::from_hms_micro(23, 37, 4, 2200);
+        let roughly_half_past_eleven = NaiveTime::from_hms_micro_opt(23, 37, 4, 2200).unwrap();
         let query = select(sql::<Time>("'23:37:04.002200'").eq(roughly_half_past_eleven));
         assert!(query.get_result::<bool>(connection).unwrap());
     }
@@ -296,7 +358,7 @@ mod tests {
     #[test]
     fn times_of_day_decode_correctly() {
         let connection = &mut connection();
-        let midnight = NaiveTime::from_hms(0, 0, 0);
+        let midnight = NaiveTime::from_hms_opt(0, 0, 0).unwrap();
         let valid_midnight_formats = &[
             "00:00",
             "00:00:00",
@@ -329,11 +391,11 @@ mod tests {
             );
         }
 
-        let noon = NaiveTime::from_hms(12, 0, 0);
+        let noon = NaiveTime::from_hms_opt(12, 0, 0).unwrap();
         let query = select(sql::<Time>("'12:00:00'"));
         assert_eq!(Ok(noon), query.get_result::<NaiveTime>(connection));
 
-        let roughly_half_past_eleven = NaiveTime::from_hms_micro(23, 37, 4, 2200);
+        let roughly_half_past_eleven = NaiveTime::from_hms_micro_opt(23, 37, 4, 2200).unwrap();
         let query = select(sql::<Time>("'23:37:04.002200'"));
         assert_eq!(
             Ok(roughly_half_past_eleven),
@@ -344,19 +406,19 @@ mod tests {
     #[test]
     fn dates_encode_correctly() {
         let connection = &mut connection();
-        let january_first_2000 = NaiveDate::from_ymd(2000, 1, 1);
+        let january_first_2000 = NaiveDate::from_ymd_opt(2000, 1, 1).unwrap();
         let query = select(date("2000-01-01").eq(january_first_2000));
         assert!(query.get_result::<bool>(connection).unwrap());
 
-        let distant_past = NaiveDate::from_ymd(0, 4, 11);
+        let distant_past = NaiveDate::from_ymd_opt(0, 4, 11).unwrap();
         let query = select(date("0000-04-11").eq(distant_past));
         assert!(query.get_result::<bool>(connection).unwrap());
 
-        let january_first_2018 = NaiveDate::from_ymd(2018, 1, 1);
+        let january_first_2018 = NaiveDate::from_ymd_opt(2018, 1, 1).unwrap();
         let query = select(date("2018-01-01").eq(january_first_2018));
         assert!(query.get_result::<bool>(connection).unwrap());
 
-        let distant_future = NaiveDate::from_ymd(9999, 1, 8);
+        let distant_future = NaiveDate::from_ymd_opt(9999, 1, 8).unwrap();
         let query = select(date("9999-01-08").eq(distant_future));
         assert!(query.get_result::<bool>(connection).unwrap());
     }
@@ -364,25 +426,25 @@ mod tests {
     #[test]
     fn dates_decode_correctly() {
         let connection = &mut connection();
-        let january_first_2000 = NaiveDate::from_ymd(2000, 1, 1);
+        let january_first_2000 = NaiveDate::from_ymd_opt(2000, 1, 1).unwrap();
         let query = select(date("2000-01-01"));
         assert_eq!(
             Ok(january_first_2000),
             query.get_result::<NaiveDate>(connection)
         );
 
-        let distant_past = NaiveDate::from_ymd(0, 4, 11);
+        let distant_past = NaiveDate::from_ymd_opt(0, 4, 11).unwrap();
         let query = select(date("0000-04-11"));
         assert_eq!(Ok(distant_past), query.get_result::<NaiveDate>(connection));
 
-        let january_first_2018 = NaiveDate::from_ymd(2018, 1, 1);
+        let january_first_2018 = NaiveDate::from_ymd_opt(2018, 1, 1).unwrap();
         let query = select(date("2018-01-01"));
         assert_eq!(
             Ok(january_first_2018),
             query.get_result::<NaiveDate>(connection)
         );
 
-        let distant_future = NaiveDate::from_ymd(9999, 1, 8);
+        let distant_future = NaiveDate::from_ymd_opt(9999, 1, 8).unwrap();
         let query = select(date("9999-01-08"));
         assert_eq!(
             Ok(distant_future),
@@ -393,29 +455,37 @@ mod tests {
     #[test]
     fn datetimes_decode_correctly() {
         let connection = &mut connection();
-        let january_first_2000 = NaiveDate::from_ymd(2000, 1, 1).and_hms(1, 1, 1);
+        let january_first_2000 = NaiveDate::from_ymd_opt(2000, 1, 1)
+            .unwrap()
+            .and_hms_opt(1, 1, 1)
+            .unwrap();
         let query = select(datetime("2000-01-01 01:01:01.000000"));
         assert_eq!(
             Ok(january_first_2000),
             query.get_result::<NaiveDateTime>(connection)
         );
 
-        let distant_past = NaiveDate::from_ymd(0, 4, 11).and_hms(2, 2, 2);
+        let distant_past = NaiveDate::from_ymd_opt(0, 4, 11)
+            .unwrap()
+            .and_hms_opt(2, 2, 2)
+            .unwrap();
         let query = select(datetime("0000-04-11 02:02:02.000000"));
         assert_eq!(
             Ok(distant_past),
             query.get_result::<NaiveDateTime>(connection)
         );
 
-        let january_first_2018 = NaiveDate::from_ymd(2018, 1, 1);
+        let january_first_2018 = NaiveDate::from_ymd_opt(2018, 1, 1).unwrap();
         let query = select(date("2018-01-01"));
         assert_eq!(
             Ok(january_first_2018),
             query.get_result::<NaiveDate>(connection)
         );
 
-        let distant_future = NaiveDate::from_ymd(9999, 1, 8)
-            .and_hms(23, 59, 59)
+        let distant_future = NaiveDate::from_ymd_opt(9999, 1, 8)
+            .unwrap()
+            .and_hms_opt(23, 59, 59)
+            .unwrap()
             .with_nanosecond(100_000)
             .unwrap();
         let query = select(sql::<Timestamp>("'9999-01-08 23:59:59.000100'"));
@@ -428,22 +498,33 @@ mod tests {
     #[test]
     fn datetimes_encode_correctly() {
         let connection = &mut connection();
-        let january_first_2000 = NaiveDate::from_ymd(2000, 1, 1).and_hms(0, 0, 0);
+        let january_first_2000 = NaiveDate::from_ymd_opt(2000, 1, 1)
+            .unwrap()
+            .and_hms_opt(0, 0, 0)
+            .unwrap();
         let query = select(datetime("2000-01-01 00:00:00.000000").eq(january_first_2000));
         assert!(query.get_result::<bool>(connection).unwrap());
 
-        let distant_past = NaiveDate::from_ymd(0, 4, 11).and_hms(20, 00, 20);
+        let distant_past = NaiveDate::from_ymd_opt(0, 4, 11)
+            .unwrap()
+            .and_hms_opt(20, 00, 20)
+            .unwrap();
         let query = select(datetime("0000-04-11 20:00:20.000000").eq(distant_past));
         assert!(query.get_result::<bool>(connection).unwrap());
 
-        let january_first_2018 = NaiveDate::from_ymd(2018, 1, 1)
-            .and_hms(12, 00, 00)
+        let january_first_2018 = NaiveDate::from_ymd_opt(2018, 1, 1)
+            .unwrap()
+            .and_hms_opt(12, 00, 00)
+            .unwrap()
             .with_nanosecond(500_000)
             .unwrap();
         let query = select(sql::<Timestamp>("'2018-01-01 12:00:00.000500'").eq(january_first_2018));
         assert!(query.get_result::<bool>(connection).unwrap());
 
-        let distant_future = NaiveDate::from_ymd(9999, 1, 8).and_hms(0, 0, 0);
+        let distant_future = NaiveDate::from_ymd_opt(9999, 1, 8)
+            .unwrap()
+            .and_hms_opt(0, 0, 0)
+            .unwrap();
         let query = select(datetime("9999-01-08 00:00:00.000000").eq(distant_future));
         assert!(query.get_result::<bool>(connection).unwrap());
     }
@@ -464,7 +545,7 @@ mod tests {
         .execute(conn)
         .unwrap();
 
-        let time: DateTime<Utc> = Utc.ymd(1970, 1, 1).and_hms_milli(0, 0, 0, 0);
+        let time: DateTime<Utc> = Utc.with_ymd_and_hms(1970, 1, 1, 0, 0, 0).single().unwrap();
 
         crate::insert_into(test_insert_timestamptz_into_table_as_text::table)
             .values(vec![(
@@ -543,11 +624,11 @@ mod tests {
             .select(test_query_timestamptz_column_with_between::timestamp_with_tz)
             .filter(
                 test_query_timestamptz_column_with_between::timestamp_with_tz
-                    .gt(Utc.ymd(1970, 1, 1).and_hms_milli(0, 0, 0, 0)),
+                    .gt(Utc.with_ymd_and_hms(1970, 1, 1, 0, 0, 0).single().unwrap()),
             )
             .filter(
                 test_query_timestamptz_column_with_between::timestamp_with_tz
-                    .lt(Utc.ymd(1970, 1, 1).and_hms_milli(0, 0, 4, 0)),
+                    .lt(Utc.with_ymd_and_hms(1970, 1, 1, 0, 0, 4).single().unwrap()),
             )
             .count()
             .get_result::<_>(conn);
@@ -558,9 +639,14 @@ mod tests {
     fn unix_epoch_encodes_correctly_with_timezone() {
         let connection = &mut connection();
         // West one hour is negative offset
-        let time = FixedOffset::west(3600)
-            .ymd(1970, 1, 1)
-            .and_hms_milli(0, 0, 0, 1);
+        let time = FixedOffset::west_opt(3600)
+            .unwrap()
+            .with_ymd_and_hms(1970, 1, 1, 0, 0, 0)
+            .single()
+            .unwrap()
+            // 1ms
+            .with_nanosecond(1_000_000)
+            .unwrap();
         let query = select(sql::<TimestamptzSqlite>("'1970-01-01 01:00:00.001+00:00'").eq(time));
         assert!(query.get_result::<bool>(connection).unwrap());
     }
@@ -568,12 +654,18 @@ mod tests {
     #[test]
     fn unix_epoch_encodes_correctly_with_utc_timezone() {
         let connection = &mut connection();
-        let time: DateTime<Utc> = Utc.ymd(1970, 1, 1).and_hms_milli(0, 0, 0, 1);
+        let time: DateTime<Utc> = Utc
+            .with_ymd_and_hms(1970, 1, 1, 0, 0, 0)
+            .single()
+            .unwrap()
+            // 1ms
+            .with_nanosecond(1_000_000)
+            .unwrap();
         let query = select(sql::<TimestamptzSqlite>("'1970-01-01 00:00:00.001+00:00'").eq(time));
         assert!(query.get_result::<bool>(connection).unwrap());
 
         // and without millisecond
-        let time: DateTime<Utc> = Utc.ymd(1970, 1, 1).and_hms_milli(0, 0, 0, 0);
+        let time: DateTime<Utc> = Utc.with_ymd_and_hms(1970, 1, 1, 0, 0, 0).single().unwrap();
         let query = select(sql::<TimestamptzSqlite>("'1970-01-01 00:00:00+00:00'").eq(time));
         assert!(query.get_result::<bool>(connection).unwrap());
     }
@@ -581,7 +673,7 @@ mod tests {
     #[test]
     fn unix_epoch_decodes_correctly_with_utc_timezone_in_all_possible_formats() {
         let connection = &mut connection();
-        let time: DateTime<Utc> = Utc.ymd(1970, 1, 1).and_hms(0, 0, 0);
+        let time: DateTime<Utc> = Utc.with_ymd_and_hms(1970, 1, 1, 0, 0, 0).single().unwrap();
         let valid_epoch_formats = vec![
             "1970-01-01 00:00Z",
             "1970-01-01 00:00:00Z",
@@ -599,22 +691,6 @@ mod tests {
             "1970-01-01T00:00:00+00:00",
             "1970-01-01T00:00:00.000+00:00",
             "1970-01-01T00:00:00.000000+00:00",
-            "1970-01-01 00:00+01:00",
-            "1970-01-01 00:00:00+01:00",
-            "1970-01-01 00:00:00.000+01:00",
-            "1970-01-01 00:00:00.000000+01:00",
-            "1970-01-01T00:00+01:00",
-            "1970-01-01T00:00:00+01:00",
-            "1970-01-01T00:00:00.000+01:00",
-            "1970-01-01T00:00:00.000000+01:00",
-            "1970-01-01T00:00-01:00",
-            "1970-01-01T00:00:00-01:00",
-            "1970-01-01T00:00:00.000-01:00",
-            "1970-01-01T00:00:00.000000-01:00",
-            "1970-01-01T00:00-01:00",
-            "1970-01-01T00:00:00-01:00",
-            "1970-01-01T00:00:00.000-01:00",
-            "1970-01-01T00:00:00.000000-01:00",
             "2440587.5",
         ];
 
